@@ -21,7 +21,9 @@ use wasm_bindgen_test::wasm_bindgen_test;
 ))]
 use wasm_bindgen_test::wasm_bindgen_test_configure;
 
-use callbag::{map, Message};
+use callbag::{map, CallbagFn, Message};
+
+type MessagePredicate<I, O> = fn(&Message<I, O>) -> bool;
 
 #[cfg(all(
     all(target_arch = "wasm32", not(target_os = "wasi")),
@@ -36,7 +38,7 @@ wasm_bindgen_test_configure!(run_in_browser);
     wasm_bindgen_test
 )]
 fn it_maps_a_pullable_source() {
-    let upwards_expected: Vec<(fn(&Message<_, _>) -> bool, &str)> = vec![
+    let upwards_expected: Vec<(MessagePredicate<_, _>, &str)> = vec![
         (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
         (|m| matches!(m, Message::Pull), "Message::Pull"),
         (|m| matches!(m, Message::Pull), "Message::Pull"),
@@ -44,7 +46,7 @@ fn it_maps_a_pullable_source() {
         (|m| matches!(m, Message::Pull), "Message::Pull"),
     ];
     let upwards_expected: Arc<RwLock<VecDeque<_>>> = Arc::new(RwLock::new(upwards_expected.into()));
-    let downwards_expected_types: Vec<(fn(&Message<_, _>) -> bool, &str)> = vec![
+    let downwards_expected_types: Vec<(MessagePredicate<_, _>, &str)> = vec![
         (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
         (|m| matches!(m, Message::Data(_)), "Message::Data"),
         (|m| matches!(m, Message::Data(_)), "Message::Data"),
@@ -60,14 +62,13 @@ fn it_maps_a_pullable_source() {
     let make_source = move || {
         let sink_ref = Arc::new(RwLock::new(None));
         let sent = Arc::new(AtomicUsize::new(0));
-        let source_ref: Arc<RwLock<Option<Box<dyn Fn(Message<_, _>) + Send + Sync>>>> =
-            Arc::new(RwLock::new(None));
+        let source_ref: Arc<RwLock<Option<CallbagFn<_, _>>>> = Arc::new(RwLock::new(None));
         let source = {
             let source_ref = source_ref.clone();
             move |message| {
                 {
                     let upwards_expected = &*upwards_expected.read().unwrap();
-                    assert!(upwards_expected.len() > 0, "source can be pulled");
+                    assert!(!upwards_expected.is_empty(), "source can be pulled");
                 }
                 {
                     let upwards_expected = &mut *upwards_expected.write().unwrap();
@@ -114,7 +115,6 @@ fn it_maps_a_pullable_source() {
                     let sink_ref = sink_ref.read().unwrap();
                     let sink_ref = sink_ref.as_ref().unwrap();
                     sink_ref(Message::Data(30));
-                    return;
                 }
             }
         };
@@ -122,12 +122,12 @@ fn it_maps_a_pullable_source() {
             let mut source_ref = source_ref.write().unwrap();
             *source_ref = Some(Box::new(source.clone()));
         }
-        source.into()
+        source
     };
 
     let make_sink = move || {
         let talkback = Arc::new(RwLock::new(None));
-        (move |message| {
+        move |message| {
             {
                 let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
                 let et = downwards_expected_types.pop_front().unwrap();
@@ -141,7 +141,6 @@ fn it_maps_a_pullable_source() {
                 let talkback = talkback.read().unwrap();
                 let talkback = talkback.as_ref().unwrap();
                 talkback(Message::Pull);
-                return;
             } else if let Message::Data(data) = message {
                 {
                     let downwards_expected = &mut *downwards_expected.write().unwrap();
@@ -151,16 +150,14 @@ fn it_maps_a_pullable_source() {
                 let talkback = talkback.read().unwrap();
                 let talkback = talkback.as_ref().unwrap();
                 talkback(Message::Pull);
-                return;
             }
-        })
-        .into()
+        }
     };
 
     let source = make_source();
-    let mapped = map(move |x| (x as f32 * 0.1) as usize)(source);
+    let mapped = map(move |x| (x as f32 * 0.1) as usize)(source.into());
     let sink = make_sink();
-    mapped(Message::Handshake(sink));
+    mapped(Message::Handshake(sink.into()));
 }
 
 /// See <https://github.com/staltz/callbag-map/blob/b9d984b78bf4301d0525b21f928d896842e17a0a/test.js#L88-L154>
@@ -175,10 +172,10 @@ fn it_maps_a_pullable_source() {
 )]
 async fn it_maps_an_async_finite_source() {
     let (nursery, nursery_out) = Nursery::new(async_executors::AsyncStd);
-    let upwards_expected: Vec<(fn(&Message<_, _>) -> bool, &str)> =
+    let upwards_expected: Vec<(MessagePredicate<_, _>, &str)> =
         vec![(|m| matches!(m, Message::Handshake(_)), "Message::Handshake")];
     let upwards_expected: Arc<RwLock<VecDeque<_>>> = Arc::new(RwLock::new(upwards_expected.into()));
-    let downwards_expected_types: Vec<(fn(&Message<_, _>) -> bool, &str)> = vec![
+    let downwards_expected_types: Vec<(MessagePredicate<_, _>, &str)> = vec![
         (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
         (|m| matches!(m, Message::Data(_)), "Message::Data"),
         (|m| matches!(m, Message::Data(_)), "Message::Data"),
@@ -195,8 +192,7 @@ async fn it_maps_an_async_finite_source() {
         let nursery = nursery.clone();
         move || {
             let sent = Arc::new(AtomicUsize::new(0));
-            let source_ref: Arc<RwLock<Option<Box<dyn Fn(Message<_, _>) + Send + Sync>>>> =
-                Arc::new(RwLock::new(None));
+            let source_ref: Arc<RwLock<Option<CallbagFn<_, _>>>> = Arc::new(RwLock::new(None));
             let source = {
                 let source_ref = source_ref.clone();
                 move |message| {
@@ -206,7 +202,7 @@ async fn it_maps_an_async_finite_source() {
                         assert!(e.0(&message), "upwards type is expected: {}", e.1);
                     }
                     if let Message::Handshake(sink) = message {
-                        let sink = Arc::new(RwLock::new(sink));
+                        let sink = Arc::new(sink);
                         const DURATION: Duration = Duration::from_millis(100);
                         let mut interval = Delay::new(DURATION);
                         nursery
@@ -220,24 +216,20 @@ async fn it_maps_an_async_finite_source() {
                                         interval.reset(DURATION);
                                         if sent.load(AtomicOrdering::Acquire) == 0 {
                                             sent.fetch_add(1, AtomicOrdering::AcqRel);
-                                            let sink = &*sink.read().unwrap();
                                             sink(Message::Data(10));
                                             continue;
                                         }
                                         if sent.load(AtomicOrdering::Acquire) == 1 {
                                             sent.fetch_add(1, AtomicOrdering::AcqRel);
-                                            let sink = &*sink.read().unwrap();
                                             sink(Message::Data(20));
                                             continue;
                                         }
                                         if sent.load(AtomicOrdering::Acquire) == 2 {
                                             sent.fetch_add(1, AtomicOrdering::AcqRel);
-                                            let sink = &*sink.read().unwrap();
                                             sink(Message::Data(30));
                                             continue;
                                         }
                                         if sent.load(AtomicOrdering::Acquire) == 3 {
-                                            let sink = &*sink.read().unwrap();
                                             sink(Message::Terminate);
                                             break;
                                         }
@@ -245,7 +237,6 @@ async fn it_maps_an_async_finite_source() {
                                 }
                             })
                             .unwrap();
-                        let sink = &*sink.read().unwrap();
                         let source = {
                             let source_ref = &mut *source_ref.write().unwrap();
                             source_ref.take().unwrap()
@@ -258,7 +249,7 @@ async fn it_maps_an_async_finite_source() {
                 let mut source_ref = source_ref.write().unwrap();
                 *source_ref = Some(Box::new(source.clone()));
             }
-            source.into()
+            source
         }
     };
 
@@ -278,7 +269,7 @@ async fn it_maps_an_async_finite_source() {
     };
 
     let source = make_source();
-    let mapped = map(move |x| (x as f32 * 0.1) as usize)(source);
+    let mapped = map(move |x| (x as f32 * 0.1) as usize)(source.into());
     mapped(Message::Handshake(sink.into()));
 
     drop(nursery);
@@ -299,12 +290,12 @@ async fn it_maps_an_async_finite_source() {
 )]
 async fn it_returns_a_source_that_disposes_upon_upwards_end() {
     let (nursery, nursery_out) = Nursery::new(async_executors::AsyncStd);
-    let upwards_expected: Vec<(fn(&Message<_, _>) -> bool, &str)> = vec![
+    let upwards_expected: Vec<(MessagePredicate<_, _>, &str)> = vec![
         (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
         (|m| matches!(m, Message::Terminate), "Message::Terminate"),
     ];
     let upwards_expected: Arc<RwLock<VecDeque<_>>> = Arc::new(RwLock::new(upwards_expected.into()));
-    let downwards_expected_types: Vec<(fn(&Message<_, _>) -> bool, &str)> = vec![
+    let downwards_expected_types: Vec<(MessagePredicate<_, _>, &str)> = vec![
         (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
         (|m| matches!(m, Message::Data(_)), "Message::Data"),
         (|m| matches!(m, Message::Data(_)), "Message::Data"),
@@ -320,20 +311,19 @@ async fn it_returns_a_source_that_disposes_upon_upwards_end() {
         let nursery = nursery.clone();
         move || {
             let sent = Arc::new(AtomicUsize::new(0));
-            let completed = Arc::new(AtomicBool::new(false));
-            let source_ref: Arc<RwLock<Option<Box<dyn Fn(Message<_, _>) + Send + Sync>>>> =
-                Arc::new(RwLock::new(None));
+            let interval_cleared = Arc::new(AtomicBool::new(false));
+            let source_ref: Arc<RwLock<Option<CallbagFn<_, _>>>> = Arc::new(RwLock::new(None));
             let source = {
                 let source_ref = source_ref.clone();
                 move |message| {
-                    let completed = completed.clone();
+                    let interval_cleared = interval_cleared.clone();
                     {
                         let upwards_expected = &mut *upwards_expected.write().unwrap();
                         let e = upwards_expected.pop_front().unwrap();
                         assert!(e.0(&message), "upwards type is expected: {}", e.1);
                     }
                     if let Message::Handshake(sink) = message {
-                        let sink = Arc::new(RwLock::new(sink));
+                        let sink = Arc::new(sink);
                         const DURATION: Duration = Duration::from_millis(100);
                         let mut interval = Delay::new(DURATION);
                         nursery
@@ -342,24 +332,25 @@ async fn it_returns_a_source_that_disposes_upon_upwards_end() {
                                 let sent = sent.clone();
                                 let sink = sink.clone();
                                 async move {
-                                    while !completed.load(AtomicOrdering::Acquire) {
+                                    loop {
                                         Pin::new(&mut interval).await;
+                                        if interval_cleared.load(AtomicOrdering::Acquire) {
+                                            break;
+                                        }
                                         interval.reset(DURATION);
-                                        let sink = &*sink.read().unwrap();
                                         let sent = sent.fetch_add(1, AtomicOrdering::AcqRel) + 1;
                                         sink(Message::Data(sent * 10));
                                     }
                                 }
                             })
                             .unwrap();
-                        let sink = &*sink.read().unwrap();
                         let source = {
                             let source_ref = &mut *source_ref.write().unwrap();
                             source_ref.take().unwrap()
                         };
                         sink(Message::Handshake(source.into()));
-                    } else if let Message::Terminate = message {
-                        completed.store(true, AtomicOrdering::Release);
+                    } else if let Message::Error(_) | Message::Terminate = message {
+                        interval_cleared.store(true, AtomicOrdering::Release);
                     }
                 }
             };
@@ -367,13 +358,13 @@ async fn it_returns_a_source_that_disposes_upon_upwards_end() {
                 let mut source_ref = source_ref.write().unwrap();
                 *source_ref = Some(Box::new(source.clone()));
             }
-            source.into()
+            source
         }
     };
 
     let make_sink = move || {
         let talkback = Arc::new(RwLock::new(None));
-        (move |message| {
+        move |message| {
             {
                 let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
                 let et = downwards_expected_types.pop_front().unwrap();
@@ -388,19 +379,18 @@ async fn it_returns_a_source_that_disposes_upon_upwards_end() {
                 assert_eq!(data, e, "downwards data is expected: {}", e);
             }
             let downwards_expected = &*downwards_expected.read().unwrap();
-            if downwards_expected.len() == 0 {
+            if downwards_expected.is_empty() {
                 let talkback = talkback.read().unwrap();
                 let talkback = talkback.as_ref().unwrap();
                 talkback(Message::Terminate);
             }
-        })
-        .into()
+        }
     };
 
     let source = make_source();
-    let mapped = map(move |x| (x as f32 * 0.1) as usize)(source);
+    let mapped = map(move |x| (x as f32 * 0.1) as usize)(source.into());
     let sink = make_sink();
-    mapped(Message::Handshake(sink));
+    mapped(Message::Handshake(sink.into()));
 
     drop(nursery);
     async_std::future::timeout(Duration::from_millis(700), nursery_out)
